@@ -31,6 +31,8 @@
 #include "exo-marshal.h"
 #include "exo-private.h"
 
+#include "gtk2-compat.h"
+
 #define             I_(string)  g_intern_static_string(string)
 
 #if defined(G_PARAM_STATIC_NAME) && defined(G_PARAM_STATIC_NICK) && defined(G_PARAM_STATIC_BLURB)
@@ -383,7 +385,6 @@ exo_tree_view_button_press_event (GtkWidget      *widget,
         selected_paths = gtk_tree_selection_get_selected_rows (selection, NULL);
     }
 
-#if GTK_CHECK_VERSION(2,9,0)
   /* Rubberbanding in GtkTreeView 2.9.0 and above is rather buggy, unfortunately, and
    * doesn't interact properly with GTKs own DnD mechanism. So we need to block all
    * dragging here when pressing the mouse button on a not yet selected row if
@@ -418,7 +419,6 @@ exo_tree_view_button_press_event (GtkWidget      *widget,
           tree_view->priv->button_release_enables_rubber_banding = TRUE;
         }
     }
-#endif
 
   /* call the parent's button press handler */
   result = (*GTK_WIDGET_CLASS (exo_tree_view_parent_class)->button_press_event) (widget, event);
@@ -431,12 +431,12 @@ exo_tree_view_button_press_event (GtkWidget      *widget,
       && path != NULL && gtk_tree_selection_path_is_selected (selection, path))
     {
       /* check if we have to restore paths */
-      if (G_LIKELY (selection->user_func == (GtkTreeSelectionFunc) exo_noop_false))
+      if (G_LIKELY (gtk_tree_selection_get_select_function (selection) == (GtkTreeSelectionFunc) exo_noop_false))
         {
           /* just reset the select function (previously set to exo_noop_false),
            * there's no clean way to do this, so what the heck.
            */
-          selection->user_func = NULL;
+          gtk_tree_selection_set_select_function (selection, NULL, NULL, NULL);
         }
       else
         {
@@ -512,7 +512,6 @@ exo_tree_view_button_release_event (GtkWidget      *widget,
         }
     }
 
-#if GTK_CHECK_VERSION(2,9,0)
   /* check if we need to re-enable drag and drop */
   if (G_LIKELY (tree_view->priv->button_release_unblocks_dnd))
     {
@@ -533,7 +532,6 @@ exo_tree_view_button_release_event (GtkWidget      *widget,
       gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (tree_view), TRUE);
       tree_view->priv->button_release_enables_rubber_banding = FALSE;
     }
-#endif
 
   /* call the parent's button release handler */
   return (*GTK_WIDGET_CLASS (exo_tree_view_parent_class)->button_release_event) (widget, event);
@@ -553,7 +551,6 @@ exo_tree_view_motion_notify_event (GtkWidget      *widget,
   /* check if the event occurred on the tree view internal window and we are in single-click mode */
   if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (tree_view)) && tree_view->priv->single_click)
     {
-#if GTK_CHECK_VERSION(2,9,0)
       /* check if we're doing a rubberband selection right now (which means DnD is blocked) */
       if (G_UNLIKELY (tree_view->priv->button_release_unblocks_dnd))
         {
@@ -564,7 +561,6 @@ exo_tree_view_motion_notify_event (GtkWidget      *widget,
           gdk_window_set_cursor (event->window, NULL);
         }
       else
-#endif
         {
           /* determine the path at the event coordinates */
           if (!gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), event->x, event->y, &path, &column, NULL, NULL))
@@ -654,7 +650,7 @@ exo_tree_view_leave_notify_event (GtkWidget        *widget,
     }
 
   /* reset the cursor for the tree view internal window */
-  if (GTK_WIDGET_REALIZED (tree_view))
+  if (gtk_widget_get_realized (GTK_WIDGET (tree_view)))
     gdk_window_set_cursor (gtk_tree_view_get_bin_window (GTK_TREE_VIEW (tree_view)), NULL);
 
   /* the next button-release-event should not activate */
@@ -700,7 +696,7 @@ exo_tree_view_move_cursor (GtkTreeView    *view,
     }
 
   /* reset the cursor for the tree view internal window */
-  if (GTK_WIDGET_REALIZED (tree_view))
+  if (gtk_widget_get_realized (GTK_WIDGET (tree_view)))
     gdk_window_set_cursor (gtk_tree_view_get_bin_window (GTK_TREE_VIEW (tree_view)), NULL);
 
   /* call the parent's handler */
@@ -722,10 +718,10 @@ exo_tree_view_single_click_timeout (gpointer user_data)
   GList             *rows;
   GList             *lp;
 
-  GDK_THREADS_ENTER ();
+  //GDK_THREADS_ENTER ();  //sfm not needed because called from g_idle?
 
   /* verify that we are in single-click mode, have focus and a hover path */
-  if (GTK_WIDGET_HAS_FOCUS (tree_view) && tree_view->priv->single_click && tree_view->priv->hover_path != NULL)
+  if (gtk_widget_has_focus (GTK_WIDGET (tree_view)) && tree_view->priv->single_click && tree_view->priv->hover_path != NULL)
     {
       /* transform the hover_path to a tree iterator */
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
@@ -735,7 +731,10 @@ exo_tree_view_single_click_timeout (gpointer user_data)
           gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &cursor_path, &cursor_column);
 
           /* be sure the row is fully visible */
-          gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view), tree_view->priv->hover_path, cursor_column, FALSE, 0.0f, 0.0f);
+          gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view),
+                            tree_view->priv->hover_path, 
+                            0, //sfm was cursor_column - caused horizontal scroll
+                            FALSE, 0.0f, 0.0f);
 
           /* determine the selection and change it appropriately */
           selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
@@ -814,7 +813,7 @@ exo_tree_view_single_click_timeout (gpointer user_data)
         }
     }
 
-  GDK_THREADS_LEAVE ();
+  //GDK_THREADS_LEAVE ();
 
   return FALSE;
 }
