@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include "glib-utils.h"
+#include <glib/gi18n.h>
 
 #include "pref-dialog.h"
 #include "settings.h"
@@ -29,6 +30,9 @@
 #include "main-window.h"
 #include "ptk-file-browser.h"
 #include "desktop.h"
+#include "ptk-location-view.h"
+
+#include "gtk2-compat.h"
 
 typedef struct _FMPrefDlg FMPrefDlg;
 struct _FMPrefDlg
@@ -46,10 +50,10 @@ struct _FMPrefDlg
     GtkWidget* small_icon_size;
     GtkWidget* tool_icon_size;
     GtkWidget* single_click;
-    GtkWidget* right_design;
     GtkWidget* use_si_prefix;
-    GtkWidget* rubberband;
+    //GtkWidget* rubberband;
     GtkWidget* root_bar;
+    GtkWidget* drag_action;
 
     /* Interface tab */
     GtkWidget* always_show_tabs;
@@ -91,14 +95,15 @@ static const int tool_icon_sizes[] = {
     GTK_ICON_SIZE_BUTTON,
     GTK_ICON_SIZE_DND,
     GTK_ICON_SIZE_DIALOG };
-static const int big_icon_sizes[] = { 96, 72, 64, 48, 36, 32, 24, 20 };
-static const int small_icon_sizes[] = { 48, 36, 32, 24, 20, 16, 12 };
+static const int big_icon_sizes[] = { 96, 72, 64, 48, 36, 32, 24, 22 };
+static const int small_icon_sizes[] = { 48, 36, 32, 24, 22, 16, 12 };
 static const char* date_formats[] =
 {
     "%Y-%m-%d %H:%M",
     "%Y-%m-%d",
     "%Y-%m-%d %H:%M:%S"
 };
+static const int drag_actions[] = { 0, 1, 2, 3 };
 /*
 static void
 on_show_desktop_toggled( GtkToggleButton* show_desktop, GtkWidget* desktop_page )
@@ -137,7 +142,7 @@ static void on_update_img_preview( GtkFileChooser *chooser, GtkImage* img )
 static void
 dir_unload_thumbnails( const char* path, VFSDir* dir, gpointer user_data )
 {
-    vfs_dir_unload_thumbnails( dir, (gboolean)user_data );
+    vfs_dir_unload_thumbnails( dir, GPOINTER_TO_INT( user_data ) );
 }
 
 static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
@@ -155,7 +160,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
     //gboolean show_desktop;
     gboolean show_wallpaper;
     gboolean single_click;
-    gboolean rubberband;
+    //gboolean rubberband;
     gboolean root_bar;
     gboolean root_set_change = FALSE;
     WallpaperMode wallpaper_mode;
@@ -167,10 +172,10 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
     const GList* l;
     PtkFileBrowser* file_browser;
     gboolean use_si_prefix;
-    gboolean right_design;
     GtkNotebook* notebook;
     int cur_tabx, p;
     FMMainWindow* a_window;
+    char* str;
 
     GtkWidget * tab_label;
     /* interface settings */
@@ -215,7 +220,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
                 a_window = FM_MAIN_WINDOW( l->data );
                 for ( p = 1; p < 5; p++ )
                 {
-                    notebook = a_window->panel[p-1];
+                    notebook = GTK_NOTEBOOK( a_window->panel[p-1] );
                     n = gtk_notebook_get_n_pages( notebook );
                     if ( always_show_tabs )
                         gtk_notebook_set_show_tabs( notebook, TRUE );
@@ -234,7 +239,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
                 a_window = FM_MAIN_WINDOW( l->data );
                 for ( p = 1; p < 5; p++ )
                 {
-                    notebook = a_window->panel[p-1];
+                    notebook = GTK_NOTEBOOK( a_window->panel[p-1] );
                     n = gtk_notebook_get_n_pages( notebook );
                     for ( i = 0; i < n; ++i )
                     {
@@ -379,7 +384,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
                 a_window = FM_MAIN_WINDOW( l->data );
                 for ( p = 1; p < 5; p++ )
                 {
-                    notebook = a_window->panel[p-1];
+                    notebook = GTK_NOTEBOOK( a_window->panel[p-1] );
                     n = gtk_notebook_get_n_pages( notebook );
                     for ( i = 0; i < n; ++i )
                     {
@@ -412,25 +417,32 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
 
             /* unload old thumbnails (icons of *.desktop files will be unloaded here, too)  */
             if( big_icon != app_settings.big_icon_size )
-                vfs_dir_foreach( (GHFunc)dir_unload_thumbnails, (gpointer)TRUE );
+                vfs_dir_foreach( (GHFunc)dir_unload_thumbnails, GINT_TO_POINTER( 1 ) );
             if( small_icon != app_settings.small_icon_size )
-                vfs_dir_foreach( (GHFunc)dir_unload_thumbnails, (gpointer)FALSE );
+                vfs_dir_foreach( (GHFunc)dir_unload_thumbnails, GINT_TO_POINTER( 0 ) );
             // update all windows/all panels/all browsers
             for ( l = fm_main_window_get_all(); l; l = l->next )
             {
                 a_window = FM_MAIN_WINDOW( l->data );
                 for ( p = 1; p < 5; p++ )
                 {
-                    notebook = a_window->panel[p-1];
+                    notebook = GTK_NOTEBOOK( a_window->panel[p-1] );
                     n = gtk_notebook_get_n_pages( notebook );
                     for ( i = 0; i < n; ++i )
                     {
                         file_browser = PTK_FILE_BROWSER( gtk_notebook_get_nth_page(
                                                          notebook, i ) );
                         ptk_file_browser_update_display( file_browser );
+                        if ( file_browser->side_dir )
+                        {
+                            gtk_widget_destroy( file_browser->side_dir );
+                            file_browser->side_dir = NULL;
+                            ptk_file_browser_update_views( NULL, file_browser );
+                        }
                     }
                 }
             }
+            // update desktop icons
             if ( big_icon != app_settings.big_icon_size )
             {
                 app_settings.big_icon_size = big_icon;
@@ -438,6 +450,9 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
             }
             app_settings.big_icon_size = big_icon;
             app_settings.small_icon_size = small_icon;
+
+            update_bookmark_icons();            
+            update_volume_icons();            
         }
 
         if ( tool_icon != app_settings.tool_icon_size )
@@ -447,16 +462,33 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
             rebuild_toolbar_all_windows( 1, NULL );
         }
 
-	    /* unit settings changed? */
-	    use_si_prefix = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data->use_si_prefix ) );
+        /* unit settings changed? */
+        gboolean need_refresh = FALSE;
+        use_si_prefix = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data->use_si_prefix ) );
         if( use_si_prefix != app_settings.use_si_prefix )
+        {
             app_settings.use_si_prefix = use_si_prefix;
+            need_refresh = TRUE;
+        }
 
-	    /* right_design settings changed? */
-	    right_design = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data->right_design ) );
-        if( right_design != app_settings.right_design )
-            app_settings.right_design = right_design;
-
+        // date format
+        char* etext = g_strdup( gtk_entry_get_text( GTK_ENTRY( gtk_bin_get_child(
+                                            GTK_BIN( data->date_format ) ) ) ) );
+        if ( g_strcmp0( etext, xset_get_s( "date_format" ) ) )
+        {
+            if ( etext[0] == '\0' )
+                xset_set( "date_format", "s", "%Y-%m-%d %H:%M" );
+            else
+                xset_set( "date_format", "s", etext );
+            g_free( etext );
+            if ( app_settings.date_format )
+                g_free( app_settings.date_format );
+            app_settings.date_format = g_strdup( xset_get_s( "date_format" ) );
+            need_refresh = TRUE;
+        }
+        if ( need_refresh )
+            main_window_refresh_all();
+        
         /* single click changed? */
         single_click = gtk_toggle_button_get_active( (GtkToggleButton*)data->single_click );
         if( single_click != app_settings.single_click )
@@ -468,14 +500,14 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
                 a_window = FM_MAIN_WINDOW( l->data );
                 for ( p = 1; p < 5; p++ )
                 {
-                    notebook = a_window->panel[p-1];
+                    notebook = GTK_NOTEBOOK( a_window->panel[p-1] );
                     n = gtk_notebook_get_n_pages( notebook );
                     for ( i = 0; i < n; ++i )
                     {
                         file_browser = PTK_FILE_BROWSER( gtk_notebook_get_nth_page(
                                                          notebook, i ) );
                         ptk_file_browser_set_single_click( file_browser, app_settings.single_click );
-                        /* ptk_file_browser_set_single_click_timeout( file_browser, 400 ); */
+                        /* ptk_file_browser_set_single_click_timeout( file_browser, SINGLE_CLICK_TIMEOUT ); */
                     }
                 }
             }
@@ -488,6 +520,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
         app_settings.no_confirm = !gtk_toggle_button_get_active(
                                             (GtkToggleButton*)data->confirm_delete );
 
+        /*
         rubberband = gtk_toggle_button_get_active(
                                             (GtkToggleButton*)data->rubberband );
         if ( !!rubberband != !!xset_get_b( "rubberband" ) )
@@ -495,7 +528,8 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
             xset_set_b( "rubberband", rubberband );
             main_window_rubberband_all();
         }
-            
+        */
+        
         root_bar = gtk_toggle_button_get_active(
                                             (GtkToggleButton*)data->root_bar );
         if ( !!root_bar != !!xset_get_b( "root_bar" ) )
@@ -504,16 +538,10 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
             main_window_root_bar_all();
         }
         
-        char* etext = g_strdup( gtk_entry_get_text( GTK_ENTRY( gtk_bin_get_child(
-                                            GTK_BIN( data->date_format ) ) ) ) );
-        if ( etext[0] == '\0' )
-            xset_set( "date_format", "s", "%Y-%m-%d %H:%M" );
-        else
-            xset_set( "date_format", "s", etext );
-        g_free( etext );
-        if ( app_settings.date_format )
-            g_free( app_settings.date_format );
-        app_settings.date_format = g_strdup( xset_get_s( "date_format" ) );
+        char* s = g_strdup_printf( "%d",
+                    gtk_combo_box_get_active( GTK_COMBO_BOX( data->drag_action ) ) );
+        xset_set( "drag_action", "x", s );
+        g_free( s );
 
         //MOD su command
         char* custom_su = NULL;
@@ -556,8 +584,8 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
         xset_set( "editor", "s", gtk_entry_get_text( GTK_ENTRY( data->editor ) ) );
         xset_set_b( "editor", gtk_toggle_button_get_active(
                                             (GtkToggleButton*)data->editor_terminal ) );
-        char* root_editor = gtk_entry_get_text( GTK_ENTRY( data->root_editor ) );
-        char* old_root_editor = xset_get_s( "root_editor" );
+        const char* root_editor = gtk_entry_get_text( GTK_ENTRY( data->root_editor ) );
+        const char* old_root_editor = xset_get_s( "root_editor" );
         if ( !old_root_editor )
         {
             if ( root_editor[0] != '\0' )
@@ -582,18 +610,25 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
 
         //MOD terminal
         char* old_terminal = xset_get_s( "main_terminal" );
-        char* terminal = gtk_combo_box_get_active_text( GTK_COMBO_BOX( data->terminal ) );
-        if ( !old_terminal && terminal[0] != '\0' )
+        char* terminal = gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT( data->terminal ) );
+        g_strstrip( terminal );
+        if ( g_strcmp0( terminal, old_terminal ) )
         {
-            xset_set( "main_terminal", "s", terminal );
+            xset_set( "main_terminal", "s", terminal[0] == '\0' ? NULL : terminal );
             root_set_change = TRUE;
         }
-        else if ( strcmp( terminal, old_terminal ) )
+        // report missing terminal
+        if ( str = strchr( terminal, ' ' ) )
+            str[0] = '\0';
+        str = g_find_program_in_path( terminal );
+        if ( !str )
         {
-            xset_set( "main_terminal", "s", terminal );
-            root_set_change = TRUE;
+            str = g_strdup_printf( "Unable to find terminal program '%s'", terminal );
+            ptk_show_error( GTK_WINDOW( dlg ), "Error", str );
         }
-          
+        g_free( str );
+        g_free( terminal );
+        
         /* save to config file */
 
         char* err_msg = save_settings( NULL );
@@ -630,8 +665,8 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
             if ( root_set_change )
             {
                 // task
-                xset_msg_dialog( GTK_WINDOW( dlg ), 0, "Save Root Settings", NULL, 0, "You will now be asked for your root password to save the root settings for this user to a file in /etc/spacefm/  Supplying the password in the next window is recommended to improve your security.", NULL );
-                PtkFileTask* task = ptk_file_exec_new( "Save Root Settings", NULL, NULL,
+                xset_msg_dialog( GTK_WIDGET( dlg ), 0, _("Save Root Settings"), NULL, 0, _("You will now be asked for your root password to save the root settings for this user to a file in /etc/spacefm/  Supplying the password in the next window is recommended.  Because SpaceFM runs some commands as root via su, these settings are best protected by root."), NULL, NULL );
+                PtkFileTask* task = ptk_file_exec_new( _("Save Root Settings"), NULL, NULL,
                                                                     NULL );
                 task->task->exec_command = g_strdup_printf( "echo" );
                 task->task->exec_as_user = g_strdup_printf( "root" );
@@ -651,7 +686,7 @@ static void on_response( GtkDialog* dlg, int response, FMPrefDlg* user_data )
 void on_date_format_changed( GtkComboBox *widget, FMPrefDlg* data )
 {
     char buf[ 128 ];
-    char* etext;
+    const char* etext;
 
     time_t now = time( NULL );
     etext = gtk_entry_get_text( GTK_ENTRY( gtk_bin_get_child( GTK_BIN( data->date_format ) ) ) );
@@ -662,11 +697,14 @@ void on_date_format_changed( GtkComboBox *widget, FMPrefDlg* data )
 void on_show_thumbnail_toggled( GtkWidget* widget, FMPrefDlg* data )
 {
     gtk_widget_set_sensitive( data->max_thumb_size,
-                    gtk_toggle_button_get_active( data->show_thumbnail ) );
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->show_thumbnail ) ) );
     gtk_widget_set_sensitive( data->thumb_label1, 
-                    gtk_toggle_button_get_active( data->show_thumbnail ) );
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->show_thumbnail ) ) );
     gtk_widget_set_sensitive( data->thumb_label2, 
-                    gtk_toggle_button_get_active( data->show_thumbnail ) );
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->show_thumbnail ) ) );
 }
 
 gboolean fm_edit_preference( GtkWindow* parent, int page )
@@ -681,11 +719,20 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
     {
         GtkTreeModel* model;
         // this invokes GVFS-RemoteVolumeMonitor via IsSupported
+#if GTK_CHECK_VERSION(2, 24, 0)
+        GtkBuilder* builder = _gtk_builder_new_from_file( PACKAGE_UI_DIR "/prefdlg2.ui", NULL );
+#else
         GtkBuilder* builder = _gtk_builder_new_from_file( PACKAGE_UI_DIR "/prefdlg.ui", NULL );
+#endif
+        if ( !builder )
+            return FALSE;
         pcmanfm_ref();
 
         data = g_new0( FMPrefDlg, 1 );
         dlg = (GtkWidget*)gtk_builder_get_object( builder, "dlg" );
+        if ( parent )
+            gtk_window_set_transient_for( GTK_WINDOW( dlg ), parent );
+        xset_set_window_icon( GTK_WINDOW( dlg ) );
 
         ptk_dialog_fit_small_screen( GTK_DIALOG( dlg ) );
         data->dlg = dlg;
@@ -705,14 +752,14 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
         data->small_icon_size = (GtkWidget*)gtk_builder_get_object( builder, "small_icon_size" );
         data->tool_icon_size = (GtkWidget*)gtk_builder_get_object( builder, "tool_icon_size" );
         data->single_click = (GtkWidget*)gtk_builder_get_object( builder, "single_click" );
-	    data->right_design = (GtkWidget*)gtk_builder_get_object( builder, "right_design" );
-	    data->use_si_prefix = (GtkWidget*)gtk_builder_get_object( builder, "use_si_prefix" );
-        data->rubberband = (GtkWidget*)gtk_builder_get_object( builder, "rubberband" );
-	    data->root_bar = (GtkWidget*)gtk_builder_get_object( builder, "root_bar" );
+        data->use_si_prefix = (GtkWidget*)gtk_builder_get_object( builder, "use_si_prefix" );
+        //data->rubberband = (GtkWidget*)gtk_builder_get_object( builder, "rubberband" );
+        data->root_bar = (GtkWidget*)gtk_builder_get_object( builder, "root_bar" );
+        data->drag_action = (GtkWidget*)gtk_builder_get_object( builder, "drag_action" );
 
         model = GTK_TREE_MODEL( gtk_list_store_new( 1, G_TYPE_STRING ) );
         gtk_combo_box_set_model( GTK_COMBO_BOX( data->terminal ), model );
-        gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( data->terminal ), 0 );
+        gtk_combo_box_set_entry_text_column( GTK_COMBO_BOX( data->terminal ), 0 );
         g_object_unref( model );
 
         //if ( '\0' == ( char ) app_settings.encoding[ 0 ] )
@@ -747,7 +794,7 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
 
         for ( i = 0; i < G_N_ELEMENTS( terminal_programs ); ++i )
         {
-            gtk_combo_box_append_text ( GTK_COMBO_BOX( data->terminal ), terminal_programs[ i ] );
+            gtk_combo_box_text_append_text ( GTK_COMBO_BOX_TEXT( data->terminal ), terminal_programs[ i ] );
         }
 
         char* terminal = xset_get_s( "main_terminal" );
@@ -760,7 +807,7 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
             }
             if ( i >= G_N_ELEMENTS( terminal_programs ) )
             { /* Found */
-                gtk_combo_box_prepend_text ( GTK_COMBO_BOX( data->terminal ), terminal );
+                gtk_combo_box_text_prepend_text ( GTK_COMBO_BOX_TEXT( data->terminal ), terminal );
                 i = 0;
             }
             gtk_combo_box_set_active( GTK_COMBO_BOX( data->terminal ), i );
@@ -829,17 +876,29 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
         gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON( data->click_exec ),
                                                         !app_settings.no_execute );
 
-        gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON( data->rubberband ),
-                                                        xset_get_b( "rubberband" ) );
+        //gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON( data->rubberband ),
+        //                                                xset_get_b( "rubberband" ) );
 
         gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON( data->root_bar ),
                                                         xset_get_b( "root_bar" ) );
         gtk_widget_set_sensitive( data->root_bar, geteuid() == 0 );
 
+        int drag_action = xset_get_int( "drag_action", "x" );
+        int drag_action_set = 0;
+        for ( i = 0; i < G_N_ELEMENTS( drag_actions ); ++i )
+        {
+            if ( drag_actions[ i ] == drag_action )
+            {
+                drag_action_set = i;
+                break;
+            }
+        }
+        gtk_combo_box_set_active( GTK_COMBO_BOX( data->drag_action ), drag_action_set );
+
+
         /* Setup 'Desktop' tab */
 
-    	gtk_toggle_button_set_active( (GtkToggleButton*)data->use_si_prefix, app_settings.use_si_prefix );
-    	gtk_toggle_button_set_active( (GtkToggleButton*)data->right_design, app_settings.right_design );
+        gtk_toggle_button_set_active( (GtkToggleButton*)data->use_si_prefix, app_settings.use_si_prefix );
 /*
         data->show_desktop = (GtkWidget*)gtk_builder_get_object( builder, "show_desktop" );
         g_signal_connect( data->show_desktop, "toggled",
@@ -921,7 +980,8 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
 #endif
         if ( custom_gsu )
         {
-            GtkListStore* gsu_list = gtk_combo_box_get_model( data->gsu_command );
+            GtkListStore* gsu_list = GTK_LIST_STORE( gtk_combo_box_get_model( 
+                                        GTK_COMBO_BOX( data->gsu_command ) ) );
             gtk_list_store_prepend( gsu_list, &it );
             gtk_list_store_set( GTK_LIST_STORE( gsu_list ), &it, 0, custom_gsu, -1 );
         }
@@ -955,11 +1015,11 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
                                                                 "label_date_disp" );
         model = GTK_TREE_MODEL( gtk_list_store_new( 1, G_TYPE_STRING ) );
         gtk_combo_box_set_model( GTK_COMBO_BOX( data->date_format ), model );
-        gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( data->date_format ), 0 );
+        gtk_combo_box_set_entry_text_column( GTK_COMBO_BOX( data->date_format ), 0 );
         g_object_unref( model );
         for ( i = 0; i < G_N_ELEMENTS( date_formats ); ++i )
         {
-            gtk_combo_box_append_text ( GTK_COMBO_BOX( data->date_format ), date_formats[ i ] );
+            gtk_combo_box_text_append_text ( GTK_COMBO_BOX_TEXT( data->date_format ), date_formats[ i ] );
         }
         char* date_s = xset_get_s( "date_format" );
         if ( date_s )
@@ -971,7 +1031,7 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
             }
             if ( i >= G_N_ELEMENTS( date_formats ) )
             {
-                gtk_combo_box_prepend_text ( GTK_COMBO_BOX( data->date_format ), date_s );
+                gtk_combo_box_text_prepend_text ( GTK_COMBO_BOX_TEXT( data->date_format ), date_s );
                 i = 0;
             }
             gtk_combo_box_set_active( GTK_COMBO_BOX( data->date_format ), i );
@@ -1003,10 +1063,9 @@ gboolean fm_edit_preference( GtkWindow* parent, int page )
         g_object_unref( builder );
     }
 
-    if( parent )
-        gtk_window_set_transient_for( GTK_WINDOW( data->dlg ), parent );
     gtk_notebook_set_current_page( (GtkNotebook*)data->notebook, page );
 
     gtk_window_present( (GtkWindow*)data->dlg );
+    return TRUE;
 }
 
