@@ -88,7 +88,7 @@ static void add_list_item( GtkListStore* list, VFSAppDesktop* desktop )
                         COL_APP_NAME, vfs_app_desktop_get_disp_name( desktop ),
                         COL_DESKTOP_FILE, vfs_app_desktop_get_name( desktop ), -1 );
     if ( icon )
-        gdk_pixbuf_unref( icon );
+        g_object_unref( icon );
 }
 
 static GtkTreeModel* create_model_from_mime_type( VFSMimeType* mime_type )
@@ -125,12 +125,23 @@ gboolean on_cmdline_keypress( GtkWidget *widget,
                                  GdkEventKey *event,
                                  GtkNotebook* notebook )
 {
-    gtk_widget_set_sensitive( notebook, gtk_entry_get_text_length(
+    gtk_widget_set_sensitive( GTK_WIDGET( notebook ), gtk_entry_get_text_length(
                                             (GtkEntry*)widget ) == 0 );
     return FALSE;
 }
 
-GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
+void on_view_row_activated ( GtkTreeView *tree_view,
+                             GtkTreePath *path,
+                             GtkTreeViewColumn* col,
+                             GtkWidget* dlg )
+{
+    GtkBuilder* builder = (GtkBuilder*)g_object_get_data(G_OBJECT(dlg), "builder");
+    GtkWidget* ok = (GtkWidget*)gtk_builder_get_object( builder, "okbutton");
+    gtk_button_clicked( GTK_BUTTON( ok ) );
+}
+
+GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type,
+                                                        gboolean no_default )
 {
     GtkBuilder* builder = _gtk_builder_new_from_file( PACKAGE_UI_DIR "/appchooserdlg.ui", NULL );
     GtkWidget * dlg = (GtkWidget*)gtk_builder_get_object( builder, "dlg" );
@@ -143,6 +154,8 @@ GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
     
     g_object_set_data_full( G_OBJECT(dlg), "builder", builder, (GDestroyNotify)g_object_unref );
 
+    xset_set_window_icon( GTK_WINDOW( dlg ) );
+
     gtk_dialog_set_alternative_button_order( GTK_DIALOG(dlg), GTK_RESPONSE_OK, GTK_RESPONSE_CANCEL, -1 );
     ptk_dialog_fit_small_screen( GTK_DIALOG(dlg) );
 
@@ -160,15 +173,16 @@ GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
         g_free( mime_desc );
     }
     /* Don't set default handler for directories and files with unknown type */
-    if ( 0 == strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_UNKNOWN ) ||
+    if ( no_default ||
+         0 == strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_UNKNOWN ) ||
          0 == strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_DIRECTORY ) )
     {
         gtk_widget_hide( (GtkWidget*)gtk_builder_get_object( builder, "set_default" ) );
     }
 
     view = GTK_TREE_VIEW( (GtkWidget*)gtk_builder_get_object( builder, "recommended_apps" ) );
-    entry = GTK_TREE_VIEW( (GtkWidget*)gtk_builder_get_object( builder, "cmdline" ) );
-    notebook = GTK_TREE_VIEW( (GtkWidget*)gtk_builder_get_object( builder, "notebook" ) );
+    entry = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "cmdline" ) );
+    notebook = GTK_NOTEBOOK( (GtkWidget*)gtk_builder_get_object( builder, "notebook" ) );
 
     model = create_model_from_mime_type( mime_type );
     gtk_tree_view_set_model( view, model );
@@ -184,8 +198,18 @@ GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
     g_signal_connect( (GtkWidget*)gtk_builder_get_object( builder, "browse_btn"),
                                     "clicked",
                                     G_CALLBACK(on_browse_btn_clicked), dlg );
+    g_signal_connect ( G_OBJECT( view ), "row_activated",
+                                    G_CALLBACK ( on_view_row_activated ),
+                                    dlg );
 
     gtk_window_set_transient_for( GTK_WINDOW( dlg ), parent );
+    
+    if ( no_default )
+    {
+        // select All Apps tab
+        gtk_widget_show( dlg );
+        gtk_notebook_next_page( notebook );
+    }
     return dlg;
 }
 
@@ -211,12 +235,12 @@ static void on_load_all_apps_finish( VFSAsyncTask* task, gboolean is_cancelled,
     gtk_tree_view_set_model( view, model );
     g_object_unref( model );
 
-    gdk_window_set_cursor( dlg->window, NULL );
+    gdk_window_set_cursor( gtk_widget_get_window ( dlg ), NULL );
 }
 
 void
 on_notebook_switch_page ( GtkNotebook *notebook,
-                          GtkNotebookPage *page,
+                          GtkWidget *page,
                           guint page_num,
                           gpointer user_data )
 {
@@ -237,7 +261,7 @@ on_notebook_switch_page ( GtkNotebook *notebook,
             init_list_view( view );
             gtk_widget_grab_focus( GTK_WIDGET( view ) );
             busy = gdk_cursor_new_for_display( gtk_widget_get_display(GTK_WIDGET( view )), GDK_WATCH );
-            gdk_window_set_cursor( GTK_WIDGET( gtk_widget_get_toplevel(GTK_WIDGET(view)) )->window, busy );
+            gdk_window_set_cursor( gtk_widget_get_window( GTK_WIDGET( gtk_widget_get_toplevel(GTK_WIDGET(view)) ) ), busy );
             gdk_cursor_unref( busy );
 
             list = gtk_list_store_new( N_COLS, GDK_TYPE_PIXBUF,
@@ -247,6 +271,9 @@ on_notebook_switch_page ( GtkNotebook *notebook,
             g_object_set_data( G_OBJECT(dlg), "task", task );
             g_signal_connect( task, "finish", G_CALLBACK(on_load_all_apps_finish), dlg );
             vfs_async_task_execute( task );
+            g_signal_connect ( G_OBJECT( view ), "row_activated",
+                                    G_CALLBACK ( on_view_row_activated ),
+                                    dlg );
         }
     }
 }
@@ -259,7 +286,7 @@ on_notebook_switch_page ( GtkNotebook *notebook,
 */
 gchar* app_chooser_dialog_get_selected_app( GtkWidget* dlg )
 {
-    const gchar * app = NULL;
+    gchar * app = NULL;
     GtkBuilder* builder = (GtkBuilder*)g_object_get_data(G_OBJECT(dlg), "builder");
     GtkEntry* entry = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "cmdline" ) );
     GtkNotebook* notebook;
@@ -270,7 +297,7 @@ gchar* app_chooser_dialog_get_selected_app( GtkWidget* dlg )
     GtkTreeModel* model;
     GtkTreeIter it;
 
-    app = gtk_entry_get_text( entry );
+    app = (char*)gtk_entry_get_text( entry );
     if ( app && *app )
     {
         return g_strdup( app );
@@ -321,6 +348,8 @@ on_browse_btn_clicked ( GtkButton *button,
                                                   NULL );
     GtkBuilder* builder = (GtkBuilder*)g_object_get_data(G_OBJECT(parent), "builder");
 
+    xset_set_window_icon( GTK_WINDOW( dlg ) );
+
     gtk_file_chooser_set_current_folder ( GTK_FILE_CHOOSER ( dlg ),
                                           "/usr/bin" );
     if ( gtk_dialog_run( GTK_DIALOG( dlg ) ) == GTK_RESPONSE_OK )
@@ -329,7 +358,7 @@ on_browse_btn_clicked ( GtkButton *button,
         if( filename )
         {
             entry = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "cmdline" ) );
-            notebook = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "notebook" ) );
+            notebook = GTK_NOTEBOOK( (GtkWidget*)gtk_builder_get_object( builder, "notebook" ) );
             /* FIXME: path shouldn't be hard-coded */
             if( g_str_has_prefix( filename, app_path )
                 && g_str_has_suffix( filename, ".desktop" ) )
@@ -343,10 +372,10 @@ on_browse_btn_clicked ( GtkButton *button,
                 gtk_entry_set_text( entry, filename );
             }
             g_free ( filename );
-            gtk_widget_set_sensitive( notebook, gtk_entry_get_text_length(
-                                            entry ) == 0 );
-            gtk_widget_grab_focus( entry );
-            gtk_editable_set_position( entry, -1 );
+            gtk_widget_set_sensitive( GTK_WIDGET( notebook ), 
+                                gtk_entry_get_text_length( entry ) == 0 );
+            gtk_widget_grab_focus( GTK_WIDGET( entry ) );
+            gtk_editable_set_position( GTK_EDITABLE( entry ), -1 );
         }
     }
     gtk_widget_destroy( dlg );
@@ -355,9 +384,11 @@ on_browse_btn_clicked ( GtkButton *button,
 static void on_dlg_response( GtkDialog* dlg, int id, gpointer user_data )
 {
     VFSAsyncTask* task;
-    
-    int width = GTK_WIDGET( dlg ) ->allocation.width;
-    int height = GTK_WIDGET( dlg ) ->allocation.height;
+    GtkAllocation allocation;
+
+    gtk_widget_get_allocation ( GTK_WIDGET( dlg ), &allocation );
+    int width = allocation.width;
+    int height = allocation.height;
     if ( width && height )
     {
         char* str = g_strdup_printf( "%d", width );
@@ -379,7 +410,11 @@ static void on_dlg_response( GtkDialog* dlg, int id, gpointer user_data )
         task = (VFSAsyncTask*)g_object_get_data( G_OBJECT(dlg), "task" );
         if( task )
         {
+printf("spacefm: app-chooser.c -> vfs_async_task_cancel\n");
+            // see note in vfs-async-task.c: vfs_async_task_real_cancel()
+            GDK_THREADS_LEAVE(); 
             vfs_async_task_cancel( task );
+            GDK_THREADS_ENTER();
             /* The GtkListStore will be freed in "finish" handler of task - on_load_all_app_finish(). */
             g_object_unref( task );
         }
@@ -388,12 +423,14 @@ static void on_dlg_response( GtkDialog* dlg, int id, gpointer user_data )
 }
 
 gchar* ptk_choose_app_for_mime_type( GtkWindow* parent,
-                                           VFSMimeType* mime_type )
+                                           VFSMimeType* mime_type,
+                                           gboolean no_default )
 {
     GtkWidget * dlg;
-    const gchar* app = NULL;
+    gchar* app = NULL;
+    gchar* custom = NULL;
 
-    dlg = app_chooser_dialog_new( parent, mime_type );
+    dlg = app_chooser_dialog_new( parent, mime_type, no_default );
 
     g_signal_connect( dlg, "response",  G_CALLBACK(on_dlg_response), NULL );
 
@@ -411,7 +448,9 @@ gchar* ptk_choose_app_for_mime_type( GtkWindow* parent,
             else if ( strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_UNKNOWN )
                       && strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_DIRECTORY ))
             {
-                vfs_mime_type_add_action( mime_type, app, NULL );
+                vfs_mime_type_add_action( mime_type, app, &custom );
+                g_free( app );
+                app = custom;
             }
         }
     }
