@@ -39,13 +39,13 @@ static void clipboard_get_data ( GtkClipboard *clipboard,
 
     list = g_string_sized_new( 8192 );
 
-    if ( selection_data->target == gnome_target )
+    if ( gtk_selection_data_get_target ( selection_data ) == gnome_target )
     {
         action = clipboard_action == GDK_ACTION_MOVE ? "cut\n" : "copy\n";
         g_string_append( list, action );
         use_uri = TRUE;
     }
-    else if ( selection_data->target == uri_list_target )
+    else if ( gtk_selection_data_get_target ( selection_data ) == uri_list_target )
         use_uri = TRUE;
 
     for ( l = clipboard_file_list; l; l = l->next )
@@ -61,13 +61,15 @@ static void clipboard_get_data ( GtkClipboard *clipboard,
         g_string_append( list, file_name );
         g_free( file_name );
 
-        if ( selection_data->target != uri_list_target )
+        if ( gtk_selection_data_get_target ( selection_data ) != uri_list_target )
             g_string_append_c( list, '\n' );
         else
             g_string_append( list, "\r\n" );
     }
 
-    gtk_selection_data_set ( selection_data, selection_data->target, 8,
+    gtk_selection_data_set ( selection_data, 
+                             gtk_selection_data_get_target ( selection_data ),
+                             8,
                              ( guchar* ) list->str, list->len + 1 );
     /* g_debug( "clipboard data:\n%s\n\n", list->str ); */
     g_string_free( list, TRUE );
@@ -163,30 +165,36 @@ void ptk_clipboard_cut_or_copy_files( const char* working_dir,
     GtkClipboard * clip = gtk_clipboard_get( GDK_SELECTION_CLIPBOARD );
     GtkTargetList* target_list = gtk_target_list_new( NULL, 0 );
     GList* target;
-    gint i, n_targets;
+    gint n_targets;
+    GtkTargetEntry* new_target;
     GtkTargetEntry* targets;
-    GtkTargetPair* pair;
     GList *l;
     VFSFileInfo* file;
     char* file_path;
     GList* file_list = NULL;
 
     gtk_target_list_add_text_targets( target_list, 0 );
-    n_targets = g_list_length( target_list->list ) + 2;
-
-    targets = g_new0( GtkTargetEntry, n_targets );
-    target = target_list->list;
-    for ( i = 0; target; ++i, target = g_list_next( target ) )
-    {
-        pair = ( GtkTargetPair* ) target->data;
-        targets[ i ].target = gdk_atom_name ( pair->target );
-    }
-    targets[ i ].target = "x-special/gnome-copied-files";
-    targets[ i + 1 ].target = "text/uri-list";
+    targets = gtk_target_table_new_from_list( target_list, &n_targets );
+    n_targets += 2;
+    targets = g_renew( GtkTargetEntry, targets, n_targets );
+    #if 0
+    new_target = gtk_target_entry_new( "x-special/gnome-copied-files", 0, 0 );
+    #else
+    new_target = g_new0( GtkTargetEntry, 1 );
+    new_target->target = "x-special/gnome-copied-files";
+    #endif
+    g_memmove( &(targets[ n_targets - 2 ]), new_target, sizeof (GtkTargetEntry));
+    #if 0
+    new_target = gtk_target_entry_new( "text/uri-list", 0, 0 );
+    #else
+    new_target = g_new0( GtkTargetEntry, 1 );
+    new_target->target = "text/uri-list";
+    #endif
+    g_memmove( &(targets[ n_targets - 1 ]), new_target, sizeof (GtkTargetEntry));
 
     gtk_target_list_unref ( target_list );
 
-    for ( l = files; l; l = l->next )
+    for ( l = g_list_last( files ); l; l = l->prev )  //sfm was reverse order
     {
         file = ( VFSFileInfo* ) l->data;
         file_path = g_build_filename( working_dir,
@@ -205,6 +213,58 @@ void ptk_clipboard_cut_or_copy_files( const char* working_dir,
     clipboard_action = copy ? GDK_ACTION_COPY : GDK_ACTION_MOVE;
 }
 
+void ptk_clipboard_copy_file_list( char** path, gboolean copy )
+{
+    GtkClipboard * clip = gtk_clipboard_get( GDK_SELECTION_CLIPBOARD );
+    GtkTargetList* target_list = gtk_target_list_new( NULL, 0 );
+    GList* target;
+    gint n_targets;
+    GtkTargetEntry* new_target;
+    GtkTargetEntry* targets;
+    GList *l;
+    VFSFileInfo* file;
+    char** file_path;
+    GList* file_list = NULL;
+    
+    gtk_target_list_add_text_targets( target_list, 0 );
+    targets = gtk_target_table_new_from_list( target_list, &n_targets );
+    n_targets += 2;
+    targets = g_renew( GtkTargetEntry, targets, n_targets );
+    #if 0
+    new_target = gtk_target_entry_new( "x-special/gnome-copied-files", 0, 0 );
+    #else
+    new_target = g_new0( GtkTargetEntry, 1 );
+    new_target->target = "x-special/gnome-copied-files";
+    #endif
+    g_memmove( &(targets[ n_targets - 2 ]), new_target, sizeof (GtkTargetEntry));
+    #if 0
+    new_target = gtk_target_entry_new( "text/uri-list", 0, 0 );
+    #else
+    new_target = g_new0( GtkTargetEntry, 1 );
+    new_target->target = "text/uri-list";
+    #endif
+    g_memmove( &(targets[ n_targets - 1 ]), new_target, sizeof (GtkTargetEntry));
+
+    gtk_target_list_unref ( target_list );
+
+    file_path = path;
+    while ( *file_path )
+    {
+        if ( *file_path[0] == '/' )
+            file_list = g_list_append( file_list, g_strdup( *file_path ) );
+        file_path++;
+    }
+
+    gtk_clipboard_set_with_data ( clip, targets, n_targets,
+                                  clipboard_get_data,
+                                  clipboard_clean_data,
+                                  NULL );
+    g_free( targets );
+
+    clipboard_file_list = file_list;
+    clipboard_action = copy ? GDK_ACTION_COPY : GDK_ACTION_MOVE;
+}
+
 void ptk_clipboard_paste_files( GtkWindow* parent_win,
                                 const char* dest_dir, GtkTreeView* task_view )
 {
@@ -224,11 +284,15 @@ void ptk_clipboard_paste_files( GtkWindow* parent_win,
     sel_data = gtk_clipboard_wait_for_contents( clip, gnome_target );
     if ( sel_data )
     {
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
 
-        uri_list_str = ( char* ) sel_data->data;
-        if ( 0 == strncmp( ( char* ) sel_data->data, "cut", 3 ) )
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
+        if ( 0 == strncmp( ( char* ) gtk_selection_data_get_data( sel_data ), "cut", 3 ) )
             action = VFS_FILE_TASK_MOVE;
         else
             action = VFS_FILE_TASK_COPY;
@@ -243,11 +307,15 @@ void ptk_clipboard_paste_files( GtkWindow* parent_win,
     {
         uri_list_target = gdk_atom_intern( "text/uri-list", FALSE );
         sel_data = gtk_clipboard_wait_for_contents( clip, uri_list_target );
-        if ( ! sel_data )
-            return ;
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
-        uri_list_str = ( char* ) sel_data->data;
+        if ( !sel_data )
+            return;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
 
         if ( clipboard_action == GDK_ACTION_MOVE )
             action = VFS_FILE_TASK_MOVE;
@@ -268,7 +336,10 @@ void ptk_clipboard_paste_files( GtkWindow* parent_win,
             ++puri;
         }
         g_strfreev( uri_list );
-        gtk_selection_data_free( sel_data );
+
+        //sfm
+        if ( files )
+            files = g_list_reverse( files );
 
         /*
         * If only one item is selected and the item is a
@@ -280,9 +351,10 @@ void ptk_clipboard_paste_files( GtkWindow* parent_win,
                                   files,
                                   dest_dir,
                                   GTK_WINDOW( parent_win ),
-                                  task_view );
+                                  GTK_WIDGET( task_view ) );
         ptk_file_task_run( task );
     }
+    gtk_selection_data_free( sel_data );
 }
 
 
@@ -306,10 +378,14 @@ void ptk_clipboard_paste_links( GtkWindow* parent_win,
     sel_data = gtk_clipboard_wait_for_contents( clip, gnome_target );
     if ( sel_data )
     {
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
-
-        uri_list_str = ( char* ) sel_data->data;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
+        
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
         action = VFS_FILE_TASK_LINK;
         if ( uri_list_str )
         {
@@ -321,11 +397,15 @@ void ptk_clipboard_paste_links( GtkWindow* parent_win,
     {
         uri_list_target = gdk_atom_intern( "text/uri-list", FALSE );
         sel_data = gtk_clipboard_wait_for_contents( clip, uri_list_target );
-        if ( ! sel_data )
-            return ;
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
-        uri_list_str = ( char* ) sel_data->data;
+        if ( !sel_data )
+            return;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
         action = VFS_FILE_TASK_LINK;
     }
 
@@ -339,15 +419,19 @@ void ptk_clipboard_paste_links( GtkWindow* parent_win,
             ++puri;
         }
         g_strfreev( uri_list );
-        gtk_selection_data_free( sel_data );
+
+        //sfm
+        if ( files )
+            files = g_list_reverse( files );
 
         task = ptk_file_task_new( action,
                                   files,
                                   dest_dir,
                                   GTK_WINDOW( parent_win ),
-                                  task_view );
+                                  task_view ? GTK_WIDGET( task_view ) : NULL );
         ptk_file_task_run( task );
     }
+    gtk_selection_data_free( sel_data );
 }
 
 void ptk_clipboard_paste_targets( GtkWindow* parent_win,
@@ -372,10 +456,14 @@ void ptk_clipboard_paste_targets( GtkWindow* parent_win,
     sel_data = gtk_clipboard_wait_for_contents( clip, gnome_target );
     if ( sel_data )
     {
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
 
-        uri_list_str = ( char* ) sel_data->data;
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
         action = VFS_FILE_TASK_COPY;
         if ( uri_list_str )
         {
@@ -387,11 +475,16 @@ void ptk_clipboard_paste_targets( GtkWindow* parent_win,
     {
         uri_list_target = gdk_atom_intern( "text/uri-list", FALSE );
         sel_data = gtk_clipboard_wait_for_contents( clip, uri_list_target );
-        if ( ! sel_data )
-            return ;
-        if ( sel_data->length <= 0 || sel_data->format != 8 )
-            return ;
-        uri_list_str = ( char* ) sel_data->data;
+        if ( !sel_data )
+            return;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return;
+        }
+        
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
         action = VFS_FILE_TASK_COPY;
     }
 
@@ -420,13 +513,16 @@ void ptk_clipboard_paste_targets( GtkWindow* parent_win,
             ++puri;
         }
         g_strfreev( uri_list );
-        gtk_selection_data_free( sel_data );
+
+        //sfm
+        if ( files )
+            files = g_list_reverse( files );
 
         task = ptk_file_task_new( action,
                                   files,
                                   dest_dir,
                                   GTK_WINDOW( parent_win ),
-                                  task_view );
+                                  GTK_WIDGET( task_view ) );
         ptk_file_task_run( task );
         
         if ( missing_targets > 0 )
@@ -437,4 +533,85 @@ void ptk_clipboard_paste_targets( GtkWindow* parent_win,
                             missing_targets > 1 ? g_strdup_printf ( "s are" ) : 
                             g_strdup_printf ( " is" ) ) );
     }
+    gtk_selection_data_free( sel_data );
+}
+
+GList* ptk_clipboard_get_file_paths( const char* cwd, gboolean* is_cut,
+                                                    gint* missing_targets )
+{
+    GtkClipboard * clip = gtk_clipboard_get( GDK_SELECTION_CLIPBOARD );
+    GdkAtom gnome_target;
+    GdkAtom uri_list_target;
+    gchar **uri_list, **puri;
+    GtkSelectionData* sel_data = NULL;
+    GList* files = NULL;
+    gchar* file_path;
+    char* uri_list_str;
+    
+    *is_cut = FALSE;
+    *missing_targets = 0;
+
+    // get files from clip
+    gnome_target = gdk_atom_intern( "x-special/gnome-copied-files", FALSE );
+    sel_data = gtk_clipboard_wait_for_contents( clip, gnome_target );
+    if ( sel_data )
+    {
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return NULL;
+        }
+        
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
+        *is_cut = ( 0 == strncmp( ( char* ) gtk_selection_data_get_data( sel_data ),
+                                                                    "cut", 3 ) );
+
+        if ( uri_list_str )
+        {
+            while ( *uri_list_str && *uri_list_str != '\n' )
+                ++uri_list_str;
+        }
+    }
+    else
+    {
+        uri_list_target = gdk_atom_intern( "text/uri-list", FALSE );
+        sel_data = gtk_clipboard_wait_for_contents( clip, uri_list_target );
+        if ( ! sel_data )
+            return NULL;
+        if ( gtk_selection_data_get_length( sel_data ) <= 0 || 
+                                gtk_selection_data_get_format( sel_data ) != 8 )
+        {
+            gtk_selection_data_free( sel_data );
+            return NULL;
+        }
+        uri_list_str = ( char* ) gtk_selection_data_get_data( sel_data );
+    }
+
+    if ( !uri_list_str )
+    {
+        gtk_selection_data_free( sel_data );
+        return NULL;
+    }
+    
+    // create file list
+    puri = uri_list = g_uri_list_extract_uris( uri_list_str );
+    while ( *puri )
+    {
+        file_path = g_filename_from_uri( *puri, NULL, NULL );
+        if ( file_path )
+        {
+            if ( g_file_test( file_path, G_FILE_TEST_EXISTS ) )             
+            {
+                files = g_list_prepend( files, file_path );
+            }
+            else
+                *missing_targets++;
+        }
+        ++puri;
+    }
+    g_strfreev( uri_list );
+    gtk_selection_data_free( sel_data );
+
+    return files;
 }
