@@ -61,7 +61,12 @@ struct _PtkFileBrowser
     int max_thumbnail;
     int n_sel_files;
     off_t sel_size;
-
+    guint sel_change_idle;
+    
+    // path bar auto seek
+    gboolean inhibit_focus;
+    char* seek_name;
+    
     /* side pane */
     GtkWidget* side_pane_buttons;
     GtkToggleToolButton* location_btn;
@@ -79,6 +84,12 @@ struct _PtkFileBrowser
     gboolean busy : 1;
     gboolean pending_drag_status : 1;
     dev_t drag_source_dev;
+    int drag_x;
+    int drag_y;
+    gboolean pending_drag_status_tree : 1;
+    dev_t drag_source_dev_tree;
+    gboolean is_drag : 1;
+    gboolean skip_release : 1;
 
     /* folder view */
     GtkWidget* folder_view;
@@ -91,31 +102,31 @@ struct _PtkFileBrowser
     
     //MOD
     int mypanel;
-    GtkNotebook* mynotebook;
-    GtkTreeView* task_view;
+    GtkWidget* mynotebook;
+    GtkWidget* task_view;
     gpointer main_window;
     int* slide_x;
     int* slide_y;
     int* slide_s;
-    GtkHBox* toolbox;
-    GtkEntry* path_bar;
-    GtkHPaned* hpane;
-    GtkVBox* side_vbox;
-    GtkHBox* side_toolbox;
-    GtkVPaned* side_vpane_top;
-    GtkVPaned* side_vpane_bottom;
-    GtkScrolledWindow* side_dir_scroll;
-    GtkScrolledWindow* side_dev_scroll;
-    GtkScrolledWindow* side_book_scroll;
-    GtkTreeView* side_book;
-    GtkTreeView* side_dir;
-    GtkTreeView* side_dev;
-    GtkStatusbar* status_bar;
+    GtkWidget* toolbox;
+    GtkWidget* path_bar;
+    GtkWidget* hpane;
+    GtkWidget* side_vbox;
+    GtkWidget* side_toolbox;
+    GtkWidget* side_vpane_top;
+    GtkWidget* side_vpane_bottom;
+    GtkWidget* side_dir_scroll;
+    GtkWidget* side_dev_scroll;
+    GtkWidget* side_book_scroll;
+    GtkWidget* side_book;
+    GtkWidget* side_dir;
+    GtkWidget* side_dev;
+    GtkWidget* status_bar;
     GtkFrame* status_frame;
     GtkLabel* status_label;
-    GtkImage* status_image;
-    GtkToolbar* toolbar;
-    GtkToolbar* side_toolbar;
+    GtkWidget* status_image;
+    GtkWidget* toolbar;
+    GtkWidget* side_toolbar;
     GtkMenuToolButton* back_menu_btn_left;
     GtkMenuToolButton* forward_menu_btn_left;
     GtkMenuToolButton* back_menu_btn_right;
@@ -128,9 +139,10 @@ struct _PtkFileBrowser
     GtkToolButton* back_btn[3]; 
     GtkToolButton* forward_btn[3]; 
     GtkToolButton* up_btn[3]; 
-    gboolean button_press : 1;
+    //gboolean button_press : 1;
     gboolean bookmark_button_press : 1;
     char* select_path;
+    char* status_bar_custom;
 };
 
 typedef enum{
@@ -157,8 +169,8 @@ struct _PtkFileBrowserClass
 
 GType ptk_file_browser_get_type ( void );
 
-GtkWidget* ptk_file_browser_new( int curpanel, GtkNotebook* notebook,
-                                                    GtkTreeView* task_view,
+GtkWidget* ptk_file_browser_new( int curpanel, GtkWidget* notebook,
+                                                    GtkWidget* task_view,
                                                     gpointer main_window,
                                                     int* slide_x,
                                                     int* slide_y,
@@ -203,7 +215,7 @@ void ptk_file_browser_update_mime_icons( PtkFileBrowser* file_browser );
 gboolean ptk_file_browser_is_busy( PtkFileBrowser* file_browser );
 
 GtkWidget* ptk_file_browser_get_folder_view( PtkFileBrowser* file_browser );
-GtkTreeView* ptk_file_browser_get_side_view( PtkFileBrowser* file_browser );
+GtkWidget* ptk_file_browser_get_side_view( PtkFileBrowser* file_browser );
 
 void ptk_file_browser_show_hidden_files( PtkFileBrowser* file_browser,
                                          gboolean show );
@@ -234,6 +246,10 @@ void ptk_file_browser_set_sort_order( PtkFileBrowser* file_browser,
 void ptk_file_browser_set_sort_type( PtkFileBrowser* file_browser,
                                      GtkSortType order );
 
+void ptk_file_browser_set_sort_extra( PtkFileBrowser* file_browser,
+                                                            const char* setname );
+void ptk_file_browser_read_sort_extra( PtkFileBrowser* file_browser );
+
 PtkFBSortOrder ptk_file_browser_get_sort_order( PtkFileBrowser* file_browser );
 
 GtkSortType ptk_file_browser_get_sort_type( PtkFileBrowser* file_browser );
@@ -259,6 +275,9 @@ void ptk_file_browser_select_all( GtkWidget* item, PtkFileBrowser* file_browser
 void ptk_file_browser_select_last( PtkFileBrowser* file_browser ); //MOD added
 void ptk_file_browser_invert_selection( GtkWidget* item, PtkFileBrowser* file_browser );
 void ptk_file_browser_unselect_all( GtkWidget* item, PtkFileBrowser* file_browser );
+void ptk_file_browser_select_pattern( GtkWidget* item, PtkFileBrowser* file_browser,
+                                                        const char* search_key ); //sfm
+void ptk_file_browser_canon( PtkFileBrowser* file_browser, const char* path );
 
 void ptk_file_browser_rename_selected_files( PtkFileBrowser* file_browser,
                                                         GList* files, char* cwd );
@@ -287,14 +306,12 @@ void ptk_file_browser_emit_open( PtkFileBrowser* file_browser,
                                  PtkOpenAction action );
 
 //MOD
-int ptk_file_browser_no_access( char* cwd, char* smode );
+int ptk_file_browser_no_access( const char* cwd, const char* smode );
 void ptk_file_browser_update_views( GtkWidget* item, PtkFileBrowser* file_browser );
 void ptk_file_browser_go_home( GtkWidget* item, PtkFileBrowser* file_browser );
 void ptk_file_browser_go_default( GtkWidget* item, PtkFileBrowser* file_browser );
 void ptk_file_browser_add_bookmark ( GtkMenuItem *menuitem, PtkFileBrowser* file_browser );
 void ptk_file_browser_find_file( GtkMenuItem *menuitem, PtkFileBrowser* file_browser );
-void ptk_file_browser_open_folder_as_root ( GtkMenuItem *menuitem,
-                                                PtkFileBrowser* file_browser );
 void on_shortcut_new_tab_activate( GtkMenuItem* item,
                                           PtkFileBrowser* file_browser );
 void on_shortcut_new_window_activate( GtkMenuItem* item,
@@ -329,7 +346,13 @@ void ptk_file_browser_on_action( PtkFileBrowser* browser, char* setname );
 GList* folder_view_get_selected_items( PtkFileBrowser* file_browser,
                                               GtkTreeModel** model );
 void ptk_file_browser_status_change( PtkFileBrowser* file_browser, gboolean panel_focus );
-
+void ptk_file_browser_select_file( PtkFileBrowser* file_browser, char* path );
+void ptk_file_browser_select_file_list( PtkFileBrowser* file_browser,
+                                                        char** filename,
+                                                        gboolean do_select );
+void ptk_file_browser_seek_path( PtkFileBrowser* file_browser, 
+                                                    const char* seek_dir,
+                                                    const char* seek_name );
 
 
 G_END_DECLS
