@@ -12,6 +12,8 @@
 #  include <config.h>
 #endif
 
+#include <libintl.h>
+
 #include "private.h"
 
 #include <gtk/gtk.h>
@@ -32,6 +34,7 @@
 #include "vfs-file-info.h"
 #include "vfs-app-desktop.h"
 #include "ptk-app-chooser.h"
+#include "main-window.h"
 
 const char* chmod_names[] =
     {
@@ -52,14 +55,19 @@ typedef struct
     char* owner_name;
     char* group_name;
 
+    GtkEntry* mtime;
+    char* orig_mtime;
+    GtkEntry* atime;
+    char* orig_atime;
+
     GtkToggleButton* chmod_btns[ N_CHMOD_ACTIONS ];
     guchar chmod_states[ N_CHMOD_ACTIONS ];
 
     GtkLabel* total_size_label;
     GtkLabel* size_on_disk_label;
     GtkLabel* count_label;
-    off_t total_size;
-    off_t size_on_disk;
+    off64_t total_size;
+    off64_t size_on_disk;
     guint total_count;
     guint total_count_dir;
     gboolean cancel;
@@ -89,12 +97,12 @@ static void calc_total_size_of_files( const char* path, FilePropertiesDialogData
     GDir * dir;
     const char* name;
     char* full_path;
-    struct stat file_stat;
+    struct stat64 file_stat;
 
     if ( data->cancel )
         return ;
 
-    if ( lstat( path, &file_stat ) )
+    if ( lstat64( path, &file_stat ) )
         return ;
 
     data->total_size += file_stat.st_size;
@@ -107,7 +115,7 @@ static void calc_total_size_of_files( const char* path, FilePropertiesDialogData
         while ( !data->cancel && ( name = g_dir_read_name( dir ) ) )
         {
             full_path = g_build_filename( path, name, NULL );
-            lstat( full_path, &file_stat );
+            lstat64( full_path, &file_stat );
             if ( S_ISDIR( file_stat.st_mode ) )
             {
                 calc_total_size_of_files( full_path, data );
@@ -157,22 +165,32 @@ gboolean on_update_labels( FilePropertiesDialogData* data )
     gdk_threads_enter();
 
     vfs_file_size_to_string( buf2, data->total_size );
-    sprintf( buf, _("%s  ( %llu bytes )"), buf2, ( guint64 ) data->total_size );
+    sprintf( buf, _("%s ( %lu bytes )"), buf2, ( guint64 ) data->total_size );
     gtk_label_set_text( data->total_size_label, buf );
 
     vfs_file_size_to_string( buf2, data->size_on_disk );
-    sprintf( buf, _("%s  ( %llu bytes )"), buf2, ( guint64 ) data->size_on_disk );
+    sprintf( buf, _("%s ( %lu bytes )"), buf2, ( guint64 ) data->size_on_disk );
     gtk_label_set_text( data->size_on_disk_label, buf );
 
     char* count;
+    char* count_dir;
     if ( data->total_count_dir )
-        count = g_strdup_printf( _("%d file%s,  %d folder%s"), data->total_count,
-                                    data->total_count > 1 ? "s" : "",
-                                    data->total_count_dir,
-                                    data->total_count_dir > 1 ? "s" : "" );
+    {
+        count_dir = g_strdup_printf( ngettext( "%d folder",
+                                               "%d folders",
+                                               data->total_count_dir ),
+                                     data->total_count_dir );
+        count = g_strdup_printf( ngettext( "%d file, %s",
+                                           "%d files, %s",
+                                           data->total_count ),
+                                 data->total_count, count_dir );
+        g_free( count_dir );
+    }
     else
-        count = g_strdup_printf( _("%d files"), data->total_count );
-    gtk_label_set_text( data->count_label, count );
+        count = g_strdup_printf( ngettext( "%d files", "%d files", 
+                                 data->total_count), data->total_count );
+ 
+     gtk_label_set_text( data->count_label, count );
     g_free( count );
 
     gdk_threads_leave();
@@ -237,7 +255,7 @@ static void on_combo_change( GtkComboBox* combo, gpointer user_data )
             VFSMimeType* mime = (VFSMimeType*)user_data;
             parent = gtk_widget_get_toplevel( GTK_WIDGET( combo ) );
             action = (char *) ptk_choose_app_for_mime_type( GTK_WINDOW(parent),
-                                                   mime );
+                                                   mime, FALSE );
             if( action )
             {
                 gboolean exist = FALSE;
@@ -273,7 +291,7 @@ static void on_combo_change( GtkComboBox* combo, gpointer user_data )
                                             1, vfs_app_desktop_get_disp_name(app),
                                             2, action, -1 );
                         if( icon )
-                            gdk_pixbuf_unref( icon );
+                            g_object_unref( icon );
                         vfs_app_desktop_unref( app );
                         exist = TRUE;
                     }
@@ -310,6 +328,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
 
     GtkWidget * dlg = (GtkWidget*)gtk_builder_get_object( builder, "dlg" );
     GtkNotebook* notebook = (GtkNotebook*)gtk_builder_get_object( builder, "notebook" );
+    xset_set_window_icon( GTK_WINDOW( dlg ) );
 
     FilePropertiesDialogData* data;
     gboolean need_calc_size = TRUE;
@@ -320,17 +339,18 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     const char* multiple_files = _( "( multiple files )" );
     const char* calculating;
     GtkWidget* name = (GtkWidget*)gtk_builder_get_object( builder, "file_name" );
+    GtkWidget* label_name = (GtkWidget*)gtk_builder_get_object( builder, "label_filename" );
     GtkWidget* location = (GtkWidget*)gtk_builder_get_object( builder, "location" );
     gtk_editable_set_editable ( GTK_EDITABLE( location ), FALSE );
+    GtkWidget* target = (GtkWidget*)gtk_builder_get_object( builder, "target" );
+    GtkWidget* label_target = (GtkWidget*)gtk_builder_get_object( builder, "label_target" );
+    gtk_editable_set_editable ( GTK_EDITABLE( target ), FALSE );
     GtkWidget* mime_type = (GtkWidget*)gtk_builder_get_object( builder, "mime_type" );
     GtkWidget* open_with = (GtkWidget*)gtk_builder_get_object( builder, "open_with" );
 
-    GtkWidget* mtime = (GtkWidget*)gtk_builder_get_object( builder, "mtime" );
-    GtkWidget* atime = (GtkWidget*)gtk_builder_get_object( builder, "atime" );
-
     char buf[ 64 ];
     char buf2[ 32 ];
-    const char* time_format = "%Y-%m-%d %H:%M";
+    const char* time_format = "%Y-%m-%d %H:%M:%S";
 
     gchar* disp_path;
     gchar* file_type;
@@ -347,7 +367,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     int width = xset_get_int( "app_dlg", "s" );
     int height = xset_get_int( "app_dlg", "z" );
     if ( width && height )
-        gtk_window_set_default_size( GTK_WINDOW( dlg ), width, height );
+        gtk_window_set_default_size( GTK_WINDOW( dlg ), width, -1 );
 
     data = g_slice_new0( FilePropertiesDialogData );
     /* FIXME: When will the data be freed??? */
@@ -366,6 +386,8 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     data->count_label = GTK_LABEL( (GtkWidget*)gtk_builder_get_object( builder, "count" ) );
     data->owner = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "owner" ) );
     data->group = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "group" ) );
+    data->mtime = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "mtime" ) );
+    data->atime = GTK_ENTRY( (GtkWidget*)gtk_builder_get_object( builder, "atime" ) );
 
     for ( i = 0; i < N_CHMOD_ACTIONS; ++i )
     {
@@ -419,7 +441,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
     if ( same_type )
     {
         mime = vfs_file_info_get_mime_type( file );
-        file_type = g_strdup_printf( "%s ( %s )",
+        file_type = g_strdup_printf( "%s\n%s",
                                      vfs_mime_type_get_description( mime ),
                                      vfs_mime_type_get_type( mime ) );
         gtk_label_set_text( GTK_LABEL( mime_type ), file_type );
@@ -482,7 +504,7 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
                                     1, vfs_app_desktop_get_disp_name(desktop),
                                     2, *action, -1 );
                 if( icon )
-                    gdk_pixbuf_unref( icon );
+                    g_object_unref( icon );
                 vfs_app_desktop_unref( desktop );
             }
         }
@@ -520,9 +542,9 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
         gtk_widget_set_sensitive( name, FALSE );
         gtk_entry_set_text( GTK_ENTRY( name ), multiple_files );
 
-        gtk_label_set_text( GTK_LABEL( mtime ), "-" );
-        gtk_label_set_text( GTK_LABEL( atime ), "-" );
-
+        data->orig_mtime = NULL;
+        data->orig_atime = NULL;
+        
         for ( i = 0; i < N_CHMOD_ACTIONS; ++i )
         {
             gtk_toggle_button_set_inconsistent ( data->chmod_btns[ i ], TRUE );
@@ -542,9 +564,15 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
             g_free( disp_name );
         }
         else
+        {
+            if ( vfs_file_info_is_dir( file ) && 
+                                            !vfs_file_info_is_symlink( file ) )
+                gtk_label_set_markup_with_mnemonic( GTK_LABEL( label_name ),
+                                                    _("<b>Folder _Name:</b>") );
             gtk_entry_set_text( GTK_ENTRY( name ),
                                 vfs_file_info_get_disp_name( file ) );
-
+        }
+        
         gtk_editable_set_editable ( GTK_EDITABLE( name ), FALSE );
 
         if ( ! vfs_file_info_is_dir( file ) )
@@ -553,26 +581,34 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
                 caculate total file size */
             need_calc_size = FALSE;
 
-            sprintf( buf, _("%s  ( %llu bytes )"),
+            sprintf( buf, _("%s  ( %lu bytes )"),
                      vfs_file_info_get_disp_size( file ),
                      ( guint64 ) vfs_file_info_get_size( file ) );
             gtk_label_set_text( data->total_size_label, buf );
 
             vfs_file_size_to_string( buf2,
                                  vfs_file_info_get_blocks( file ) * 512 );
-            sprintf( buf, _("%s  ( %llu bytes )"), buf2,
+            sprintf( buf, _("%s  ( %lu bytes )"), buf2,
                      ( guint64 ) vfs_file_info_get_blocks( file ) * 512 );
             gtk_label_set_text( data->size_on_disk_label, buf );
             
-            gtk_label_set_text( data->count_label, "1 file" );
+            gtk_label_set_text( data->count_label, _("1 file") );
         }
-        gtk_label_set_text( GTK_LABEL( mtime ),
-                            vfs_file_info_get_disp_mtime( file ) );
+        
+        // Modified / Accessed
+        //gtk_entry_set_text( GTK_ENTRY( mtime ),
+        //                    vfs_file_info_get_disp_mtime( file ) );
+        strftime( buf, sizeof( buf ),
+                  time_format, localtime( vfs_file_info_get_mtime( file ) ) );
+        gtk_entry_set_text( GTK_ENTRY( data->mtime ), buf );
+        data->orig_mtime = g_strdup( buf );
 
         strftime( buf, sizeof( buf ),
                   time_format, localtime( vfs_file_info_get_atime( file ) ) );
-        gtk_label_set_text( GTK_LABEL( atime ), buf );
+        gtk_entry_set_text( GTK_ENTRY( data->atime ), buf );
+        data->orig_atime = g_strdup( buf );
 
+        // Permissions
         owner_group = (char *) vfs_file_info_get_disp_owner( file );
         tmp = strchr( owner_group, ':' );
         data->owner_name = g_strndup( owner_group, tmp - owner_group );
@@ -588,6 +624,28 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
                 gtk_toggle_button_set_active( data->chmod_btns[ i ], data->chmod_states[ i ] );
             }
         }
+        
+        // target
+        if ( vfs_file_info_is_symlink( file ) )
+        {
+            gtk_label_set_markup_with_mnemonic( GTK_LABEL( label_name ),
+                                                    _("<b>Link _Name:</b>") );
+            disp_path = g_build_filename( dir_path, file->name, NULL );
+            char* target_path = g_file_read_link( disp_path, NULL );
+            if ( target_path )
+            {
+                if ( !g_file_test( target_path, G_FILE_TEST_EXISTS ) )
+                    gtk_label_set_text( GTK_LABEL( mime_type ),
+                                                    _("( broken link )") );
+                gtk_entry_set_text( GTK_ENTRY( target ), target_path );
+                g_free( target_path );
+            }
+            else
+                gtk_entry_set_text( GTK_ENTRY( target ), _("( read link error )") );
+            g_free( disp_path );
+            gtk_widget_show( target );
+            gtk_widget_show( label_target );
+        }
     }
 
     if ( need_calc_size )
@@ -705,9 +763,12 @@ on_dlg_response ( GtkDialog *dialog,
     char* file_path;
     GtkWidget* ask_recursive;
     VFSFileInfo* file;
+    GtkAllocation allocation;
+
+    gtk_widget_get_allocation ( GTK_WIDGET( dialog ), &allocation );
     
-    int width = GTK_WIDGET( dialog ) ->allocation.width;
-    int height = GTK_WIDGET( dialog ) ->allocation.height;
+    int width = allocation.width;
+    int height = allocation.height;
     if ( width && height )
     {
         char* str = g_strdup_printf( "%d", width );
@@ -731,9 +792,68 @@ on_dlg_response ( GtkDialog *dialog,
 
         if ( response_id == GTK_RESPONSE_OK )
         {
-            GtkWidget* open_with;
-
+            // change file dates
+            char* cmd = NULL;
+            char* quoted_time;
+            char* quoted_path;
+            const char* new_mtime = gtk_entry_get_text( data->mtime );
+            if ( !( new_mtime && new_mtime[0] ) || 
+                                !g_strcmp0( data->orig_mtime, new_mtime ) )
+                new_mtime = NULL;
+            const char* new_atime = gtk_entry_get_text( data->atime );
+            if ( !( new_atime && new_atime[0] ) || 
+                                !g_strcmp0( data->orig_atime, new_atime ) )
+                new_atime = NULL;
+            
+            if ( ( new_mtime || new_atime ) && data->file_list )
+            {
+                GString* gstr = g_string_new( NULL );
+                for ( l = data->file_list; l; l = l->next )
+                {
+                    file_path = g_build_filename( data->dir_path,
+                                                  ((VFSFileInfo*)l->data)->name,
+                                                  NULL );
+                    quoted_path = bash_quote( file_path );
+                    g_string_append_printf( gstr, " %s", quoted_path );
+                    g_free( file_path );
+                    g_free( quoted_path );
+                }
+                    
+                if ( new_mtime )
+                {
+                    quoted_time = bash_quote( new_mtime );
+                    cmd = g_strdup_printf( "touch --no-dereference --no-create -m -d %s%s",
+                                                                quoted_time,
+                                                                gstr->str );
+                }
+                if ( new_atime )
+                {
+                    quoted_time = bash_quote( new_atime );
+                    quoted_path = cmd;  // temp str
+                    cmd = g_strdup_printf( "%s%stouch --no-dereference --no-create -a -d %s%s",
+                                                                cmd ? cmd : "",
+                                                                cmd ? "\n" : "",
+                                                                quoted_time,
+                                                                gstr->str );
+                    g_free( quoted_path );
+                }
+                g_free( quoted_time );
+                g_string_free( gstr, TRUE );
+                if ( cmd )
+                {
+                    task = ptk_file_exec_new( _("Change File Date"), "/",
+                                              GTK_WIDGET( dialog ), NULL );
+                    task->task->exec_command = cmd;
+                    task->task->exec_sync = TRUE;
+                    task->task->exec_export = FALSE;
+                    task->task->exec_show_output = TRUE;
+                    task->task->exec_show_error = TRUE;
+                    ptk_file_task_run( task );
+                }
+            }
+        
             /* Set default action for mimetype */
+            GtkWidget* open_with;
             if( ( open_with = (GtkWidget*)g_object_get_data( G_OBJECT(dialog), "open_with" ) ) )
             {
                 GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(open_with) );
@@ -853,6 +973,8 @@ on_dlg_response ( GtkDialog *dialog,
 
         g_free( data->owner_name );
         g_free( data->group_name );
+        g_free( data->orig_mtime );
+        g_free( data->orig_atime );
         /*
          *NOTE: File operation chmod/chown will free the list when it's done,
          *and we only need to free it when there is no file operation applyed.
