@@ -9,6 +9,10 @@
 *
 */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE  // euidaccess
+#endif
+
 #include "vfs-dir.h"
 #include "vfs-thumbnail-loader.h"
 #include "glib-mem.h"
@@ -18,6 +22,7 @@
 
 #include <fcntl.h>  /* for open() */
 #include <unistd.h> /* for read */
+#include "vfs-volume.h"
 
 static void vfs_dir_class_init( VFSDirClass* klass );
 static void vfs_dir_init( VFSDir* dir );
@@ -201,7 +206,7 @@ void vfs_dir_init( VFSDir* dir )
 void vfs_dir_finalize( GObject *obj )
 {
     VFSDir * dir = VFS_DIR( obj );
-
+//printf("vfs_dir_finalize  %s\n", dir->path );
     do{}
     while( g_source_remove_by_user_data( dir ) );
 
@@ -209,6 +214,7 @@ void vfs_dir_finalize( GObject *obj )
     {
         g_signal_handlers_disconnect_by_func( dir->task, on_list_task_finished, dir );
         /* FIXME: should we generate a "file-list" signal to indicate the dir loading was cancelled? */
+//printf("spacefm: vfs_dir_finalize -> vfs_async_task_cancel\n");
         vfs_async_task_cancel( dir->task );
         g_object_unref( dir->task );
         dir->task = NULL;
@@ -313,10 +319,13 @@ static GList* vfs_dir_find_file( VFSDir* dir, const char* file_name, VFSFileInfo
 }
 
 /* signal handlers */
-void vfs_dir_emit_file_created( VFSDir* dir, const char* file_name, VFSFileInfo* file )
+void vfs_dir_emit_file_created( VFSDir* dir, const char* file_name, gboolean force )
 {
     GList* l;
 
+    if ( !force && dir->avoid_changes )
+        return;
+
     if ( G_UNLIKELY( 0 == strcmp(file_name, dir->path) ) )
     {
         // Special Case: The directory itself was created?
@@ -373,9 +382,22 @@ void vfs_dir_emit_file_deleted( VFSDir* dir, const char* file_name, VFSFileInfo*
     }
 }
 
-void vfs_dir_emit_file_changed( VFSDir* dir, const char* file_name, VFSFileInfo* file  )
+void vfs_dir_emit_file_changed( VFSDir* dir, const char* file_name,
+                                        VFSFileInfo* file, gboolean force )
 {
     GList* l;
+//printf("vfs_dir_emit_file_changed dir=%s file_name=%s avoid=%s\n", dir->path, file_name, dir->avoid_changes ? "TRUE" : "FALSE" );
+
+    if ( !force && dir->avoid_changes )
+        return;
+
+/*  Test not needed because file won't be found in list?
+    if ( G_UNLIKELY( 0 == strcmp(file_name, dir->path) ) )
+    {
+        // Special Case: The directory itself was changed
+        return;
+    }
+*/
 
     g_mutex_lock( dir->mutex );
 
@@ -385,8 +407,18 @@ void vfs_dir_emit_file_changed( VFSDir* dir, const char* file_name, VFSFileInfo*
         file = vfs_file_info_ref( ( VFSFileInfo* ) l->data );
         if( !g_slist_find( dir->changed_files, file ) )
         {
-            // update file info the first time
-            if( G_LIKELY( update_file_info( dir, file ) ) )
+            if ( force )
+            {
+                dir->changed_files = g_slist_prepend( dir->changed_files, file );
+                if ( 0 == change_notify_timeout )
+                {
+                    change_notify_timeout = g_timeout_add_full( G_PRIORITY_LOW,
+                                                                100,
+                                                                notify_file_change,
+                                                                NULL, NULL );
+                }
+            }
+            else if( G_LIKELY( update_file_info( dir, file ) ) ) // update file info the first time
             {
                 dir->changed_files = g_slist_prepend( dir->changed_files, file );
                 if ( 0 == change_notify_timeout )
@@ -434,6 +466,13 @@ VFSDir* vfs_dir_new( const char* path )
     VFSDir * dir;
     dir = ( VFSDir* ) g_object_new( VFS_TYPE_DIR, NULL );
     dir->path = g_strdup( path );
+
+#ifdef HAVE_HAL
+    dir->avoid_changes = FALSE;
+#else
+    dir->avoid_changes = vfs_volume_dir_avoid_changes( path );
+#endif
+//printf("vfs_dir_new %s  avoid_changes=%s\n", dir->path, dir->avoid_changes ? "TRUE" : "FALSE" );
     return dir;
 }
 
@@ -481,12 +520,29 @@ static gboolean is_dir_virtual( const char* path )
 char* gethidden( const char* path )  //MOD added
 {
     // Read .hidden into string
-    char* hidden_path = g_build_filename( path, ".hidden", NULL );            
+    char* hidden_path = g_build_filename( path, ".hidden", NULL );     
+
+    // test access first because open() on missing file may cause
+    // long delay on nfs
+    int acc;
+#if defined(HAVE_EUIDACCESS)
+    acc = euidaccess( hidden_path, R_OK );
+#elif defined(HAVE_EACCESS)
+    acc = eaccess( hidden_path, R_OK );
+#else
+    acc = 0;
+#endif
+    if ( acc != 0 )
+    {       
+        g_free( hidden_path );
+        return NULL;
+    }
+
     int fd = open( hidden_path, O_RDONLY );
     g_free( hidden_path );
     if ( fd != -1 )
     {
-        struct stat s;
+        struct stat s;   // skip stat64
         if ( G_LIKELY( fstat( fd, &s ) != -1 ) )
         {
             char* buf = g_malloc( s.st_size + 1 );
@@ -715,6 +771,7 @@ void vfs_cancel_load( VFSDir* dir )
     dir->cancel = TRUE;
     if ( dir->task )
     {
+printf("spacefm: vfs_cancel_load -> vfs_async_task_cancel\n");
         vfs_async_task_cancel( dir->task );
         /* don't do g_object_unref on task here since this is done in the handler of "finish" signal. */
         dir->task = NULL;
@@ -735,7 +792,6 @@ gboolean update_file_info( VFSDir* dir, VFSFileInfo* file )
     file->name = NULL;
 
     full_path = g_build_filename( dir->path, file_name, NULL );
-
     if ( G_LIKELY( full_path ) )
     {
         if( G_LIKELY( vfs_file_info_get( file, full_path, file_name ) ) )
@@ -799,7 +855,7 @@ void update_created_files( gpointer key, gpointer data, gpointer user_data )
     char* full_path;
     VFSFileInfo* file;
     GList* ll;
-    
+
     if ( dir->created_files )
     {
         g_mutex_lock( dir->mutex );
@@ -827,8 +883,12 @@ void update_created_files( gpointer key, gpointer data, gpointer user_data )
             {
                 // file already exists in dir file_list
                 file = vfs_file_info_ref( (VFSFileInfo*)ll->data );
-                g_signal_emit( dir, signals[ FILE_CHANGED_SIGNAL ], 0, file );
-                vfs_file_info_unref( file );
+                if ( update_file_info( dir, file ) )
+                {
+                    g_signal_emit( dir, signals[ FILE_CHANGED_SIGNAL ], 0, file );
+                    vfs_file_info_unref( file );
+                }
+                // else was deleted, signaled, and unrefed in update_file_info
             }
             g_free( (char*)l->data );  // free file_name string
         }
@@ -840,15 +900,24 @@ void update_created_files( gpointer key, gpointer data, gpointer user_data )
 
 gboolean notify_file_change( gpointer user_data )
 {
-    GDK_THREADS_ENTER();
+    //GDK_THREADS_ENTER();  //sfm not needed because in main thread?
     g_hash_table_foreach( dir_hash, update_changed_files, NULL );
     g_hash_table_foreach( dir_hash, update_created_files, NULL );
     /* remove the timeout */
     change_notify_timeout = 0;
-    GDK_THREADS_LEAVE();
+    //GDK_THREADS_LEAVE();
     return FALSE;
 }
 
+void vfs_dir_flush_notify_cache()
+{
+    if ( change_notify_timeout )
+        g_source_remove( change_notify_timeout );
+    change_notify_timeout = 0;
+    g_hash_table_foreach( dir_hash, update_changed_files, NULL );
+    g_hash_table_foreach( dir_hash, update_created_files, NULL );
+}
+
 /* Callback function which will be called when monitored events happen */
 void vfs_dir_monitor_callback( VFSFileMonitor* fm,
                                VFSFileMonitorEvent event,
@@ -861,13 +930,13 @@ void vfs_dir_monitor_callback( VFSFileMonitor* fm,
     switch ( event )
     {
     case VFS_FILE_MONITOR_CREATE:
-        vfs_dir_emit_file_created( dir, file_name, NULL );
+        vfs_dir_emit_file_created( dir, file_name, FALSE );
         break;
     case VFS_FILE_MONITOR_DELETE:
         vfs_dir_emit_file_deleted( dir, file_name, NULL );
         break;
     case VFS_FILE_MONITOR_CHANGE:
-        vfs_dir_emit_file_changed( dir, file_name, NULL );
+        vfs_dir_emit_file_changed( dir, file_name, NULL, FALSE );
         break;
     default:
         g_warning("Error: unrecognized file monitor signal!");
@@ -907,6 +976,17 @@ static void on_theme_changed( GtkIconTheme *icon_theme, gpointer user_data )
     g_hash_table_foreach( dir_hash, (GHFunc)reload_icons, NULL );
 }
 
+VFSDir* vfs_dir_get_by_path_soft( const char* path )
+{
+    if ( G_UNLIKELY( !dir_hash || !path ) )
+        return NULL;
+
+    VFSDir * dir = g_hash_table_lookup( dir_hash, path );
+    if ( dir )
+        g_object_ref( dir );
+    return dir;
+}
+    
 VFSDir* vfs_dir_get_by_path( const char* path )
 {
     VFSDir * dir = NULL;
@@ -995,7 +1075,7 @@ const char* vfs_get_desktop_dir()
         g_free( def );
         if( G_LIKELY( fd != -1 ) )
         {
-            struct stat s;
+            struct stat s;   // skip stat64
             if( G_LIKELY( fstat( fd, &s ) != -1 ) )
             {
                 char* buf = g_malloc( s.st_size + 1 );
@@ -1109,3 +1189,62 @@ void vfs_dir_unload_thumbnails( VFSDir* dir, gboolean is_big )
     }
     g_mutex_unlock( dir->mutex );
 }
+
+//sfm added mime change timer
+guint mime_change_timer = 0;
+VFSDir* mime_dir = NULL;
+
+gboolean on_mime_change_timer( gpointer user_data )
+{
+    //printf("MIME-UPDATE on_timer\n" );
+    char* cmd = g_strdup_printf( "update-mime-database %s/mime",
+                                                    g_get_user_data_dir() );
+    g_spawn_command_line_async( cmd, NULL );
+    g_free( cmd );
+    cmd = g_strdup_printf( "update-desktop-database %s/applications",
+                                                    g_get_user_data_dir() );
+    g_spawn_command_line_async( cmd, NULL );
+    g_free( cmd );
+    g_source_remove( mime_change_timer );
+    mime_change_timer = 0;
+    return FALSE;
+}
+
+void mime_change( gpointer user_data )
+{
+    if ( mime_change_timer )
+    {
+        // timer is already running, so ignore request
+        //printf("MIME-UPDATE already set\n" );
+        return;
+    }
+    if ( mime_dir )
+    {
+        // update mime database in 2 seconds 
+        mime_change_timer = g_timeout_add_seconds( 2,
+                                    ( GSourceFunc ) on_mime_change_timer, NULL );
+        //printf("MIME-UPDATE timer started\n" );
+    }
+}
+
+void vfs_dir_monitor_mime()
+{
+    // start watching for changes
+    if ( mime_dir )
+        return;
+    char* path = g_build_filename( g_get_user_data_dir(), "mime/packages", NULL );
+    if ( g_file_test( path, G_FILE_TEST_IS_DIR ) )
+    {
+        mime_dir = vfs_dir_get_by_path( path );
+        if ( mime_dir )
+        {
+            g_signal_connect( mime_dir, "file-listed", G_CALLBACK( mime_change ), NULL );
+            g_signal_connect( mime_dir, "file-created", G_CALLBACK( mime_change ), NULL );
+            g_signal_connect( mime_dir, "file-deleted", G_CALLBACK( mime_change ), NULL );
+            g_signal_connect( mime_dir, "file-changed", G_CALLBACK( mime_change ), NULL );
+        }
+        //printf("MIME-UPDATE watch started\n" );
+    }
+    g_free( path );
+}
+
